package com.example.backend.orders.api;

import java.text.ParseException;

import java.util.Date;
import java.util.Map;
import java.util.stream.Collectors;

import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.example.backend.core.api.PageDto;
import com.example.backend.core.api.PageDtoMapper;
import com.example.backend.orders.model.OrderEntity;
import com.example.backend.orders.service.OrderService;
import com.example.backend.core.configuration.Constants;
import com.example.backend.core.utils.Formatter;
import com.example.backend.books.api.BookDto;
import com.example.backend.books.model.BookEntity;

import jakarta.validation.Valid;

@RestController
@RequestMapping(Constants.API_URL + "/user/{user}/order")
public class OrderController {
    private final OrderService orderService;
    private final ModelMapper modelMapper;

    public OrderController(OrderService orderService, ModelMapper modelMapper) {
        this.orderService = orderService;
        this.modelMapper = modelMapper;
    }

    private OrderDto toDto(OrderEntity entity) {
        final OrderDto dto = modelMapper.map(entity, OrderDto.class);
        dto.setDate(Formatter.format(entity.getDate()));
        dto.setSum(orderService.getSum(entity.getUser().getId(), entity.getId()));
        dto.setCount(orderService.getFullCount(entity.getUser().getId(), entity.getId()));
        dto.setBooks(orderService.getOrderBooks(entity.getUser().getId(), entity
                .getId()).stream()
                .map(orderBook -> toBookDto(orderBook.getBook()))
                .toList());
        return dto;
    }

    private BookDto toBookDto(BookEntity entity) {
        return modelMapper.map(entity, BookDto.class);
    }

    private OrderEntity toEntity(OrderDto dto) throws ParseException {
        final OrderEntity entity = modelMapper.map(dto, OrderEntity.class);
        entity.setDate(Formatter.parse(dto.getDate()));
        return entity;
    }

    @GetMapping
    public PageDto<OrderDto> getAll(
            @PathVariable(name = "user") Long userId,
            @RequestParam(name = "page", defaultValue = "0") int page,
            @RequestParam(name = "size", defaultValue = Constants.DEFAULT_PAGE_SIZE) int size) {
        return PageDtoMapper.toDto(orderService.getAll(userId, page, size), this::toDto);
    }

    @GetMapping("/{id}")
    public OrderDto get(
            @PathVariable(name = "user") Long userId,
            @PathVariable(name = "id") Long id) {
        return toDto(orderService.get(userId, id));
    }

    @PostMapping
    public OrderDto create(
            @PathVariable(name = "user") Long userId,
            @RequestBody @Valid OrderDto dto) throws ParseException {
        dto.setDate(Formatter.format(new Date()));
        Map<Long, Integer> booksIdsCount = dto.getOrderInfo().stream()
                .collect(Collectors.toMap(OrderBookDto::getBookId, OrderBookDto::getCount));
        return toDto(orderService.create(userId, toEntity(dto), booksIdsCount));
    }

    @DeleteMapping("/{id}")
    public OrderDto delete(
            @PathVariable(name = "user") Long userId,
            @PathVariable(name = "id") Long id) {
        return toDto(orderService.delete(userId, id));
    }
}
